// Include the libraries we need
#include <OneWire.h>
#include <DallasTemperature.h>
#include <Wire.h> 
#include <LiquidCrystal_I2C.h>

LiquidCrystal_I2C lcd(0x27,20,4);  // set the LCD address to 0x27 for a 16 chars and 2 line display
// Data wire is plugged into port 2 on the Arduino 4 esp
#define ONE_WIRE_BUS 2
#define TEMPERATURE_PRECISION 12
#define led 14

  //float  litre = 5;  // rated litre per minute not used in this sketch - 
  float  Dtemp = 26;  // rated tempature increase - normally found in your tankless water heater manual 
  float  Dtempa;
  float  Mtemp = 55;   // maximum temp
  float inputT;
  float inputTa;
  float inputD;
  float inputDa;
  int pulse = 254;
 
// Setup a oneWire instance to communicate with any OneWire devices (not just Maxim/Dallas temperature ICs)
OneWire oneWire(ONE_WIRE_BUS);

#include <math.h>

double Thermistor(int RawADC) {
 double Temp;
 //Temp = log(10000.0*((1024.0/RawADC-1))); // adjust 10000.0  to get accurate  temp - use other inline if temputer move in opposite direction
  Temp =log(11500.0/(1024.0/RawADC-1)); // for pull-up configuration 
 Temp = 1 / (0.001129148 + (0.000234125 + (0.0000000876741 * Temp * Temp ))* Temp );
 Temp = Temp - 273.15;            // Convert Kelvin to Celcius
 //Temp = (Temp * 9.0)/ 5.0 + 32.0; // Convert Celcius to Fahrenheit
 return Temp;
}

// Pass our oneWire reference to Dallas Temperature. 
DallasTemperature sensors(&oneWire);

// arrays to hold device addresses
DeviceAddress inputa, outputa, inputb, outputb;

void setup()
{
  analogWrite(led, 0);
  analogWriteFreq(220);// adjust this freq to get best preformance form SSR or triac (using MOC3043)
  // start serial port
  Serial.begin(115200);
  Serial.println("Solar water heater  instant water heater controller");
  lcd.init();                      // initialize the lcd 
  lcd.init();
  // Print a message to the LCD.
  lcd.backlight();
  lcd.setCursor(0,0);
  lcd.print("Solar Water Temp");
  lcd.setCursor(2,1);
  lcd.print("Controller!");


  // Start up the library
  sensors.begin();

  // locate devices on the bus
  Serial.print("Locating devices...");
  Serial.print("Found ");
  Serial.print(sensors.getDeviceCount(), DEC);
  Serial.println(" devices.");

  // report parasite power requirements
  Serial.print("Parasite power is: "); 
  if (sensors.isParasitePowerMode()) Serial.println("ON");
  else Serial.println("OFF");

  // Assign address manually. The addresses below will beed to be changed
  // to valid device addresses on your bus. Device address can be retrieved
  // by using either oneWire.search(deviceAddress) or individually via
  // sensors.getAddress(deviceAddress, index)
  //insideThermometer = { 0x28, 0x1D, 0x39, 0x31, 0x2, 0x0, 0x0, 0xF0 };
  //outsideThermometer   = { 0x28, 0x3F, 0x1C, 0x31, 0x2, 0x0, 0x0, 0x2 };

  // Search for devices on the bus and assign based on an index. Ideally,
  // you would do this to initially discover addresses on the bus and then 
  // use those addresses and manually assign them (see above) once you know 
  // the devices on your bus (and assuming they don't change).
  // 
  // method 1: by index
  if (!sensors.getAddress(inputa, 0)) Serial.println("Unable to find address for Device 0"); 
  if (!sensors.getAddress(outputa, 1)) Serial.println("Unable to find address for Device 1"); 
  // if (!sensors.getAddress(inputb, 2)) Serial.println("Unable to find address for Device 2"); 
 // if (!sensors.getAddress(outputb, 3)) Serial.println("Unable to find address for Device 3"); 

  // method 2: search()
  // search() looks for the next device. Returns 1 if a new address has been
  // returned. A zero might mean that the bus is shorted, there are no devices, 
  // or you have already retrieved all of them. It might be a good idea to 
  // check the CRC to make sure you didn't get garbage. The order is 
  // deterministic. You will always get the same devices in the same order
  //
  // Must be called before search()
  //oneWire.reset_search();
  // assigns the first address found to insideThermometer
  //if (!oneWire.search(insideThermometer)) Serial.println("Unable to find address for insideThermometer");
  // assigns the seconds address found to outsideThermometer
  //if (!oneWire.search(outsideThermometer)) Serial.println("Unable to find address for outsideThermometer");

  // show the addresses we found on the bus
  Serial.print("Device 0 Address: ");
  printAddress(inputa);
  Serial.println();

  Serial.print("Device 1 Address: ");
  printAddress(outputa);
  Serial.println();

  
  sensors.setResolution(inputa, TEMPERATURE_PRECISION);
  sensors.setResolution(outputa, TEMPERATURE_PRECISION);


  Serial.print("Device 0 Resolution: ");
  Serial.print(sensors.getResolution(inputa), DEC); 
  Serial.println();

  Serial.print("Device 1 Resolution: ");
  Serial.print(sensors.getResolution(outputa), DEC); 
  Serial.println();

}

// function to print a device address
void printAddress(DeviceAddress deviceAddress)
{
  for (uint8_t i = 0; i < 8; i++)
  {
    // zero pad the address if necessary
    if (deviceAddress[i] < 16) Serial.print("0");
    Serial.print(deviceAddress[i], HEX);
  }
}

// function to print the temperature for a device
void printTemperature(DeviceAddress deviceAddress)
{
  float tempC = sensors.getTempC(deviceAddress);
  Serial.print("Temp C: ");
  Serial.print(tempC);
  //Serial.print(" Temp F: ");
  //Serial.print(DallasTemperature::toFahrenheit(tempC));
}

// function to print a device's resolution
void printResolution(DeviceAddress deviceAddress)
{
  Serial.print("Resolution: ");
  Serial.print(sensors.getResolution(deviceAddress));
  Serial.println();    
}

// main function to print information about a device
void printData(DeviceAddress deviceAddress)
{
  Serial.print("Device Address: ");
  printAddress(deviceAddress);
  Serial.print(" ");
  printTemperature(deviceAddress);
  Serial.println();
}

/*
 * Main function, calls the temperatures in a loop.
 */
void loop()
 {

  
  // call sensors.requestTemperatures() to issue a global temperature 
  // request to all devices on the bus
  Serial.print("Requesting temperatures...");
  sensors.requestTemperatures();
  Serial.println("DONE");

 

  float tempB = sensors.getTempC(inputa);  
  float tempD = ((Thermistor(analogRead(0))));  
  inputT= Mtemp - tempB;
  inputD= inputT / Dtemp;
  
  Serial.print("digital temp ");
Serial.println(tempB);
   Serial.print("analog temp ");
Serial.println(tempD); 
  
  if (inputD <=0) { inputD = 0;}
  if (inputD > 1) { inputD = 1;}

  
 
  pulse = 256*inputD;
    if (pulse < 0) { pulse = 0;}
  if (pulse > 256) { pulse = 256;}
  pulse = (pulse*4);
  analogWrite(led, pulse);
 for (int i=0; i <= 75; i++){  // adjust resample rate get best prformance for flow rate
  tempD = ((Thermistor(analogRead(0))));
 
inputTa = (Mtemp-tempD);

 inputDa = (inputTa / Dtemp);

inputD = (inputD + inputDa);


 if (inputD <=0) { inputD = 0;}
  if (inputD > 1) { inputD = 1;}
  pulse = 256*inputD;

 if (pulse < 0) { pulse = 0;}
  if (pulse > 256) { pulse = 256;}
  pulse=  pulse*4;
  if (Mtemp < tempB) {pulse=0;}
  analogWrite(led, pulse);

  }
 
 
  
  lcd.clear();
lcd.setCursor(0,0);
  lcd.print(tempB);
  lcd.setCursor(0,1);
  lcd.print(tempD);
 lcd.setCursor(6,0);
  lcd.print(inputT); 
  lcd.setCursor(6,1);
  lcd.print(pulse);

}

